C--------------------------------------------------------------------
C The user of the OPEN-ADAS system and data (hereinafter "the User")
C accepts the following terms and conditions: 
C 
C   1. The University of Strathclyde uses all reasonable endeavours
C      to ensure the accuracy of the data provided and any
C      information given, but the University makes no warranty,
C      expressed or implied as to its accuracy, integrity, fitness
C      for purpose and shall not be responsible for the use to
C      which the data is put by the User and/or any consequences
C      arising out of any inaccuracies or omissions.
C 
C   2. Any downloaded OPEN-ADAS file is made available here for the
C      personal use of the User only.  It must not be used for any
C      commercial application, incorporated into any web site or in
C      any form re-distributed without express prior written
C      permission from the ADAS Project. In particular, but not by
C      way of limitation, it must not be:
C 
C        * inserted into a managed database structure
C        * redistributed along with a modelling or analysis code
C        * made available on a public website.
C 
C Permission for OPEN-ADAS data incorporation in non-commercial
C code/system development for scientific advance will not be
C unreasonably withheld, but will be subject to written agreement
C on a case by case basis.
C 
C The User will indemnify the University of Strathclyde and keep
C it fully and effectively indemnified against each and every
C claim made against the University as a result of the User's
C use of the data.  
C--------------------------------------------------------------------
       PROGRAM TEST
       
       IMPLICIT NONE
C------------------------------------------------------------------------
C  Purpose:
C          Test the XXDATA_12 routine and also provide a brief
C          example of its use.
C
C  Explanaton:
C          This program is distributed as part of the ADAS
C          subroutine library either via direct CVS to ADAS Project
C          members or via download by OPEN-ADAS users. It reads a
C          file called "test.dat" (distributed along with this
C          program) and prints out some of the contents. The
C          program is intended to serve as an example of using the
C          XXDATA_12 subroutine as well as testing it. A short
C          shell script called "test.sh" is used to compile and
C          test the code.
C
C  Variables:
C          All variables are documented by XXDATA_12. See either the
C          Fortran source or the PDF documentation for the routine.
C          Variables not passed into XXDATA_12 are trivial in nature.
C
C  Routines:
C          Routine    Source    Brief description
C          -------------------------------------------------------------
C          XXDATA_12  ADAS      Read complete data from an ADF12 file
C
C  Author : Allan Whiteford
C
C  Date   : 11/01/08
C       
C-----------------------------------------------------------------------

C-----------------------------------------------------------------------
C     Parameters required by XXDATA_12
C     --------------------------------        
      INTEGER     NDSTORE       , NDTEM         , NDDEN
      INTEGER     NDEIN         , NDZEF         , NDMAG
      PARAMETER ( NDSTORE = 50  , NDTEM = 12    , NDDEN = 24 )
      PARAMETER ( NDEIN   = 24  , NDZEF = 12    , NDMAG = 12 )
C-----------------------------------------------------------------------

C-----------------------------------------------------------------------
C     Input variables (i.e. non-parameter) required by XXDATA_12
C     ----------------------------------------------------------
      INTEGER      IUNIT
      CHARACTER*80 DSNAME     
C-----------------------------------------------------------------------		

C-----------------------------------------------------------------------
C     Output variables required by XXDATA_12
C     --------------------------------------
      CHARACTER*2         CINDM(NDSTORE),           CSYMB(NDSTORE)         
      CHARACTER*2         CTYPE(NDSTORE),           CZION(NDSTORE)
      CHARACTER*5         CRECVR(NDSTORE)
      CHARACTER*6         CDONOR(NDSTORE)
      CHARACTER*7         CTRANS(NDSTORE)
      CHARACTER*8         CWAVEL(NDSTORE)
      CHARACTER*10        CFILE(NDSTORE)
      INTEGER             NBSEL,                    NBMAGA(NDSTORE)
      INTEGER             NDENSA(NDSTORE),          NZEFFA(NDSTORE)
      INTEGER             NENERA(NDSTORE),          NTEMPA(NDSTORE)
      REAL*8              BMAGA(NDMAG,NDSTORE),     BMREF(NDSTORE)
      REAL*8              DENSA(NDDEN,NDSTORE),     DEREF(NDSTORE)
      REAL*8              ENERA(NDEIN,NDSTORE),     ENREF(NDSTORE)
      REAL*8              QBMAGA(NDMAG,NDSTORE)
      REAL*8              QDENSA(NDDEN,NDSTORE),    QEFREF(NDSTORE)
      REAL*8              QENERA(NDEIN,NDSTORE)
      REAL*8              QTEMPA(NDTEM,NDSTORE)
      REAL*8              QZEFFA(NDZEF,NDSTORE),    TEMPA(NDTEM,NDSTORE)
      REAL*8              TEREF(NDSTORE),           ZEFFA(NDZEF,NDSTORE)
      REAL*8              ZEREF(NDSTORE)
C-----------------------------------------------------------------------

C-----------------------------------------------------------------------
C     Variables only used by this test program
C     ----------------------------------------
      INTEGER   I,J,K
      INTEGER   ISTORE
C----------------------------------------------------------------------

C-----------------------------------------------------------------------
C     Open input file for reading
C     ---------------------------      
      IUNIT = 10
      DSNAME='test.dat'
      OPEN(UNIT = IUNIT , FILE=DSNAME , STATUS = 'OLD')
C-----------------------------------------------------------------------
       
C-----------------------------------------------------------------------
C     Pass unit number and dimension parameters to XXDATA_12 and
C     get back contents of file.
C     --------------------------      

      CALL XXDATA_12( IUNIT  , DSNAME ,
     &                NDTEM  , NDDEN  , NDEIN  , NDZEF   , NDMAG  ,
     &                NDSTORE,
     &                NBSEL  ,
     &                CSYMB  , CZION  , CWAVEL , CDONOR , CRECVR  ,
     &                CTRANS , CFILE  , CTYPE  , CINDM  ,
     &                QEFREF ,
     &                ENREF  , TEREF  , DEREF  , ZEREF   , BMREF  ,
     &                NENERA , NTEMPA , NDENSA , NZEFFA  , NBMAGA ,
     &                ENERA  , TEMPA  , DENSA  , ZEFFA   , BMAGA  ,
     &                QENERA , QTEMPA , QDENSA , QZEFFA  , QBMAGA
     &              )
C-----------------------------------------------------------------------
   
C-----------------------------------------------------------------------
C     Print out some of the contents to screen
C     ---------------------------------------- 

      PRINT *,'XXDATA_12 Test Program'      
      PRINT *,'----------------------'
      PRINT *,' '
      PRINT *,'The following is some information about the'
      PRINT *,'input file: '//DSNAME
      PRINT *,' '       

      PRINT *,'Reference parameters:'
      PRINT *, ' ( Energy Temperature Density Zeff Bmag ) = '
      PRINT *, '         ( ',ENREF(1),TEREF(1),DEREF(1),
     &                       ZEREF(1),BMREF(1), ' )'
      PRINT *,' '
      PRINT *,'Temperature Grid for fourth transition:'
      PRINT *,(TEMPA(I,4),I=1,NTEMPA(4))
      PRINT *,' '
      PRINT *,'S/XB for fourth transition as a function of'
      PRINT *,'temperature at the reference parameters:'
      PRINT *,(QTEMPA(I,4),I=1,NTEMPA(4))
      PRINT *,' '
      PRINT *,'Density Grid for fourth transition:'
      PRINT *,(DENSA(I,4),I=1,NDENSA(4))
      PRINT *,' '
      PRINT *,'S/XB for fourth transition as a function of'
      PRINT *,'density at the reference parameters:'
      PRINT *,(QDENSA(I,4),I=1,NDENSA(4))
      PRINT *,' '
      PRINT *,'Energy Grid for fourth transition:'
      PRINT *,(ENERA(I,4),I=1,NENERA(4))
      PRINT *,' '
      PRINT *,'CXRS emission for fourth transition as a function of'
      PRINT *,'beam energy at the reference parameters:'
      PRINT *,(QENERA(I,4),I=1,NENERA(4))
C-------------------------------------------------------------------

      END					  	
